@@ -285,11 +285,6 @@ def cleanup(*args, **kwargs):
     type=str,
     help="""Specify the name for this run.""",
 )
-@click.option(
-    "--allow-dirty",
-    is_flag=True,
-    help="""Allow running the experiment with a dirty working directory (uncommitted code changes).""",
-)
 @click.pass_obj
 def run(
     settings: TangoGlobalSettings,
@@ -302,7 +297,6 @@ def run(
     parallelism: Optional[int] = None,
     step_name: Optional[str] = None,
     name: Optional[str] = None,
-    allow_dirty: bool = False,
 ):
     """
     Run a tango experiment.
@@ -335,7 +329,6 @@ def run(
         step_name=step_name,
         name=name,
         called_by_executor=_CALLED_BY_EXECUTOR,
-        allow_dirty=allow_dirty,
     )
 
 
@@ -359,16 +352,22 @@ def run(
     help="Python packages or modules to import for tango components.",
     multiple=True,
 )
+@click.option(
+    "--log-level",
+    help="Set the global log level.",
+    type=click.Choice(["debug", "info", "warning", "error"], case_sensitive=False),
+    show_choices=True,
+)
 def beaker_executor_run(
     experiment: str,
     step_name: str,
     workspace_url: str,
     include_package: Optional[Sequence[str]] = None,
+    log_level: str = "debug",
 ):
     """
     This command is only used internally by the BeakerExecutor.
     """
-    from tango.common.logging import do_json_logging
     from tango.executor import Executor
 
     if include_package:
@@ -386,8 +385,7 @@ def beaker_executor_run(
     executor = Executor(workspace=workspace, include_package=include_package)
 
     # Initialize logging.
-    initialize_logging(log_level="debug", enable_cli_logs=True)
-    do_json_logging(f"step {step.name}")
+    initialize_logging(log_level=log_level, enable_cli_logs=True, file_friendly_logging=True)
 
     # Run step.
     executor.execute_step(step)
@@ -709,24 +707,12 @@ def _run(
     multicore: Optional[bool] = None,
     name: Optional[str] = None,
     called_by_executor: bool = False,
-    allow_dirty: bool = False,
 ) -> str:
-    from git import InvalidGitRepositoryError, Repo
-
     from tango.executor import Executor
     from tango.executors import MulticoreExecutor
     from tango.server.workspace_server import WorkspaceServer
     from tango.workspaces import MemoryWorkspace, default_workspace
 
-    if not called_by_executor and not allow_dirty:
-        # Make sure repository is clean, if we're in one.
-        try:
-            repo = Repo(".")
-            if repo.is_dirty():
-                raise CliRunError("You have uncommitted changes! Use --allow-dirty to force.")
-        except InvalidGitRepositoryError:
-            pass
-
     # Read params.
     params = Params.from_file(experiment, params_overrides=overrides or "")
 
@@ -750,7 +736,7 @@ def _run(
         workspace = default_workspace
 
     executor: Executor
-    if settings.executor is not None:
+    if not called_by_executor and settings.executor is not None:
         if multicore is not None:
             logger.warning(
                 "Ignoring argument 'multicore' since executor is defined in %s",
@@ -785,7 +771,7 @@ def _run(
             executor = Executor(workspace=workspace, include_package=include_package)
 
     # Initialize step graph.
-    step_graph = StepGraph.from_params(params.pop("steps", keep_as_dict=True))
+    step_graph = StepGraph.from_params(params.pop("steps"))
     sub_graph: Optional[StepGraph] = None
     params.assert_empty("'tango run'")
 
@@ -822,25 +808,26 @@ def _run(
                 "Server started at [bold]%s[/bold]", server.address_for_display(run.name)
             )
 
+        executor_output: Optional[ExecutorOutput] = None
         if step_name is not None:
             assert sub_graph is not None
             step = sub_graph[step_name]
             if step.cache_results and step in workspace.step_cache:
                 step.log_cache_hit()
             else:
-                executor.execute_sub_graph_for_step(sub_graph, step_name, run_name=run.name)
-            if not called_by_executor:
-                step.log_finished(run.name)
+                executor_output = executor.execute_sub_graph_for_step(
+                    sub_graph, step_name, run_name=run.name
+                )
         else:
             executor_output = executor.execute_step_graph(step_graph, run_name=run.name)
-            if executor_output.failed:
-                cli_logger.error(
-                    "[red]\N{ballot x} Run [bold]%s[/] finished with errors[/]", run.name
-                )
-            elif not called_by_executor:
-                cli_logger.info("[green]\N{check mark} Finished run [bold]%s[/][/]", run.name)
-            _display_run_results(run, step_graph, workspace, executor_output)
 
+        if executor_output is not None and executor_output.failed:
+            cli_logger.error("[red]\N{ballot x} Run [bold]%s[/] finished with errors[/]", run.name)
+        elif not called_by_executor:
+            cli_logger.info("[green]\N{check mark} Finished run [bold]%s[/][/]", run.name)
+
+        if not called_by_executor and executor_output is not None:
+            _display_run_results(run, step_graph, workspace, executor_output)
             if executor_output.failed:
                 raise CliRunError
 
@@ -868,29 +855,27 @@ def _display_run_results(
     table = Table(caption_style="")
     table.add_column("Step Name", justify="left", style="cyan")
     table.add_column("Status", justify="left")
-    table.add_column("Cached Result", justify="left")
+    table.add_column("Results", justify="left")
     last_cached_step: Optional[str] = None
-    for step in sorted(step_graph.values(), key=lambda step: step.name):
-        step_info = workspace.step_info(step)
-
+    for step_name in sorted(step_graph):
         status_str: str
-        if step.name in executor_output.failed:
+        result_str: str = "[grey62]N/A[/]"
+        if step_name in executor_output.failed:
             status_str = "[red]\N{ballot x} failed[/]"
-        elif step.name in executor_output.not_run:
+            execution_metadata = executor_output.failed[step_name]
+            if execution_metadata.logs_location is not None:
+                result_str = f"[cyan]{execution_metadata.logs_location}[/]"
+        elif step_name in executor_output.not_run:
             status_str = "[yellow]- not run[/]"
-        elif step.name in executor_output.successful:
+        elif step_name in executor_output.successful:
             status_str = "[green]\N{check mark} succeeded[/]"
+            execution_metadata = executor_output.successful[step_name]
+            if execution_metadata.result_location is not None:
+                result_str = f"[cyan]{execution_metadata.result_location}[/]"
         else:
             continue
 
-        result_str: str
-        if step_info.result_location is not None:
-            last_cached_step = step.name
-            result_str = f"[cyan]{step_info.result_location}[/]"
-        else:
-            result_str = "[grey62]N/A[/]"
-
-        table.add_row(step.name, status_str, result_str)
+        table.add_row(step_name, status_str, result_str)
 
     caption_parts: List[str] = []
     if executor_output.failed:
