@@ -89,9 +89,7 @@ import threading
 from contextlib import contextmanager
 from typing import ContextManager, Generator, List, Optional
 
-import jsonpickle
 import rich
-import tblib
 from rich.console import Console
 from rich.logging import RichHandler as _RichHandler
 from rich.padding import Padding
@@ -204,6 +202,8 @@ class PrefixLogFilter(logging.Filter):
         self._prefix = prefix
 
     def filter(self, record):
+        if not isinstance(record.msg, str):
+            return True
         if record.name == "tango.__main__":
             from rich.markup import escape
 
@@ -220,7 +220,7 @@ class LogRecordStreamHandler(socketserver.StreamRequestHandler):
     configured locally.
 
     Taken from
-    `the logging cookbook <https://docs.python.org/3.7/howto/logging-cookbook.html>`_.
+    `the logging cookbook <https://docs.python.org/3.8/howto/logging-cookbook.html>`_.
     """
 
     def handle(self):
@@ -259,7 +259,7 @@ class LogRecordSocketReceiver(socketserver.ThreadingTCPServer):
     Simple TCP socket-based logging receiver.
 
     Taken from
-    `the logging cookbook <https://docs.python.org/3.7/howto/logging-cookbook.html>`_.
+    `the logging cookbook <https://docs.python.org/3.8/howto/logging-cookbook.html>`_.
     """
 
     allow_reuse_address = True
@@ -554,7 +554,7 @@ def _initialize_logging(
         # Main process: set formatter and handlers, initialize logging socket and server.
         # Set up logging socket to emit log records from worker processes/threads.
         # Inspired by:
-        # https://docs.python.org/3.7/howto/logging-cookbook.html#sending-and-receiving-logging-events-across-a-network
+        # https://docs.python.org/3.8/howto/logging-cookbook.html#sending-and-receiving-logging-events-across-a-network
         _LOGGING_SERVER = LogRecordSocketReceiver(_LOGGING_HOST, 0)
         _LOGGING_PORT = _LOGGING_SERVER.server_address[1]
         os.environ[EnvVarNames.LOGGING_PORT.value] = str(_LOGGING_PORT)
@@ -678,38 +678,3 @@ def file_handler(filepath: PathOrStr) -> ContextManager[None]:
         handler.addFilter(CliFilter(filter_out=not is_cli_handler))
         handlers.append(handler)
     return insert_handlers(*handlers)
-
-
-def log_record_to_json(record: logging.LogRecord) -> str:
-    attrs = record.__dict__
-    if attrs.get("exc_info") is not None:
-        # Tracebacks cannot be pickled directly, so we need help from tblib.
-        et, ev, tb = attrs["exc_info"]
-        tb_dict = tblib.Traceback(tb).to_dict()
-        attrs["exc_info"] = (et, ev, tb_dict)
-    return jsonpickle.dumps(attrs)
-
-
-def log_record_from_json(json_record: str) -> logging.LogRecord:
-    attrs = jsonpickle.loads(json_record)
-    if attrs.get("exc_info") is not None:
-        et, ev, tb_dict = attrs["exc_info"]
-        tb = tblib.Traceback.from_dict(tb_dict)
-        attrs["exc_info"] = (et, ev, tb)
-    return logging.makeLogRecord(attrs)
-
-
-class JsonHandler(logging.Handler):
-    def emit(self, record: logging.LogRecord):
-        print(log_record_to_json(record))
-
-
-def do_json_logging(prefix: str):
-    from tango.common.tqdm import logger as tqdm_logger
-
-    root_logger = logging.getLogger()
-    for logger in (root_logger, cli_logger, tqdm_logger):
-        logger.handlers.clear()
-        handler = JsonHandler()
-        handler.addFilter(PrefixLogFilter(prefix))
-        logger.addHandler(handler)
