@@ -5,9 +5,8 @@ import threading
 import time
 import uuid
 import warnings
-from json import JSONDecodeError
 from pathlib import Path
-from typing import List, Optional, Sequence, Set, Tuple
+from typing import Dict, List, Optional, Sequence, Set, Tuple
 
 from beaker import (
     Beaker,
@@ -21,7 +20,9 @@ from beaker import (
     JobFailedError,
     TaskResources,
     TaskSpec,
+    TaskStoppedError,
 )
+from git import Git, GitCommandError, InvalidGitRepositoryError, Repo
 
 from tango.common.exceptions import (
     CancellationError,
@@ -29,9 +30,8 @@ from tango.common.exceptions import (
     ExecutorError,
     RunCancelled,
 )
-from tango.common.logging import cli_logger, log_exception, log_record_from_json
-from tango.executor import Executor, ExecutorOutput
-from tango.step import Step
+from tango.common.logging import cli_logger, log_exception
+from tango.executor import ExecutionMetadata, Executor, ExecutorOutput
 from tango.step_graph import StepGraph
 from tango.step_info import GitMetadata
 from tango.version import VERSION
@@ -42,6 +42,12 @@ from .common import Constants
 logger = logging.getLogger(__name__)
 
 
+class StepFailedError(ExecutorError):
+    def __init__(self, msg: str, experiment_url: str):
+        super().__init__(msg)
+        self.experiment_url = experiment_url
+
+
 @Executor.register("beaker")
 class BeakerExecutor(Executor):
     """
@@ -105,6 +111,9 @@ class BeakerExecutor(Executor):
     :param install_cmd: Override the command used to install your code and its dependencies
         in each Beaker job.
         For example, you could set ``install_cmd="pip install .[dev]"``.
+    :param priority: The default task priority to assign to jobs ran on Beaker.
+    :param allow_dirty: By default, the Beaker Executor requires that your git working directory has no uncommitted
+        changes. If you set this to ``True``, we skip this check.
     :param kwargs: Additional keyword arguments passed to :meth:`Beaker.from_env() <beaker.Beaker.from_env()>`.
 
     .. attention::
@@ -220,6 +229,8 @@ class BeakerExecutor(Executor):
         venv_name: Optional[str] = None,
         parallelism: Optional[int] = -1,
         install_cmd: Optional[str] = None,
+        priority: str = "normal",
+        allow_dirty: bool = False,
         **kwargs,
     ):
         # Pre-validate arguments.
@@ -268,7 +279,10 @@ class BeakerExecutor(Executor):
         self.clusters = clusters
         self.venv_name = venv_name
         self.install_cmd = install_cmd
+        self.priority = priority
+        self.allow_dirty = allow_dirty
         self._is_cancelled = threading.Event()
+        self._latest_clusters_used: List[str] = []
 
         try:
             self.github_token: str = github_token or os.environ["GITHUB_TOKEN"]
@@ -282,20 +296,45 @@ class BeakerExecutor(Executor):
         # Ensure entrypoint dataset exists.
         self._ensure_entrypoint_dataset()
 
-    def execute_step(self, step: Step) -> None:
-        raise NotImplementedError
+    def check_repo_state(self):
+        if not self.allow_dirty:
+            # Make sure repository is clean, if we're in one.
+            try:
+                # Check for uncommitted changes.
+                repo = Repo(".")
+                if repo.is_dirty():
+                    raise ExecutorError(
+                        "You have uncommitted changes! Commit your changes or use the 'allow_dirty' option."
+                    )
+
+                # Check for un-pushed commits.
+                remote_name = repo.remote().name
+                git = Git(".")
+                if git.log([f"{remote_name}..HEAD", "--not", "--remotes", "--oneline"]):
+                    raise ExecutorError(
+                        "You have unpushed changes! Push your changes or use the 'allow_dirty' option."
+                    )
+            except InvalidGitRepositoryError:
+                raise ExecutorError(
+                    "It appears you're not in a valid git repository. "
+                    "The Beaker executor requires a git repository."
+                )
+            except GitCommandError:
+                pass
 
     def execute_step_graph(
         self, step_graph: StepGraph, run_name: Optional[str] = None
     ) -> ExecutorOutput:
         import concurrent.futures
 
+        self.check_repo_state()
+
         self._is_cancelled.clear()
 
         # These will hold the final results which we'll update along the way.
-        successful: Set[str] = set()
-        failed: Set[str] = set()
-        not_run: Set[str] = set()
+        successful: Dict[str, ExecutionMetadata] = {}
+        failed: Dict[str, ExecutionMetadata] = {}
+        not_run: Dict[str, ExecutionMetadata] = {}
 
         # Keeps track of steps that are next up to run on Beaker.
         steps_to_run: Set[str] = set()
@@ -323,7 +362,7 @@ class BeakerExecutor(Executor):
                         if dependency.name not in successful and dependency.cache_results:
                             if dependency.name in failed or dependency.name in not_run:
                                 # A dependency failed or can't be run, so this step can't be run.
-                                not_run.add(step_name)
+                                not_run[step_name] = ExecutionMetadata()
                                 steps_to_run.discard(step_name)
                             break
                     else:
@@ -337,13 +376,22 @@ class BeakerExecutor(Executor):
                 try:
                     exc = future.exception()
                     if exc is None:
-                        successful.add(step_name)
+                        successful[step_name] = ExecutionMetadata(
+                            result_location=self.workspace.step_info(
+                                step_graph[step_name]
+                            ).result_location,
+                            logs_location=future.result(),
+                        )
+                    elif isinstance(exc, StepFailedError):
+                        failed[step_name] = ExecutionMetadata(logs_location=exc.experiment_url)
+                    elif isinstance(exc, ExecutorError):
+                        failed[step_name] = ExecutionMetadata()
                     else:
                         log_exception(exc, logger)
-                        failed.add(step_name)
+                        failed[step_name] = ExecutionMetadata()
                 except concurrent.futures.TimeoutError as exc:
                     log_exception(exc, logger)
-                    failed.add(step_name)
+                    failed[step_name] = ExecutionMetadata()
 
             return future_done_callback
 
@@ -393,8 +441,31 @@ class BeakerExecutor(Executor):
 
     def execute_sub_graph_for_step(
         self, step_graph: StepGraph, step_name: str, run_name: Optional[str] = None
-    ) -> None:
-        self._execute_sub_graph_for_step(step_graph, step_name)
+    ) -> ExecutorOutput:
+        self.check_repo_state()
+        try:
+            experiment_url = self._execute_sub_graph_for_step(step_graph, step_name)
+        except Exception as exc:
+            if isinstance(exc, StepFailedError):
+                return ExecutorOutput(
+                    failed={step_name: ExecutionMetadata(logs_location=exc.experiment_url)}
+                )
+            elif isinstance(exc, ExecutorError):
+                return ExecutorOutput(failed={step_name: ExecutionMetadata()})
+            else:
+                log_exception(exc, logger)
+                return ExecutorOutput(failed={step_name: ExecutionMetadata()})
+        else:
+            return ExecutorOutput(
+                successful={
+                    step_name: ExecutionMetadata(
+                        result_location=self.workspace.step_info(
+                            step_graph[step_name]
+                        ).result_location,
+                        logs_location=experiment_url,
+                    )
+                }
+            )
 
     def _check_if_cancelled(self):
         if self._is_cancelled.is_set():
@@ -405,7 +476,7 @@ class BeakerExecutor(Executor):
         step_graph: StepGraph,
         step_name: str,
         in_thread: bool = False,
-    ) -> None:
+    ) -> Optional[str]:
         if not in_thread:
             self._is_cancelled.clear()
 
@@ -416,7 +487,9 @@ class BeakerExecutor(Executor):
                 '[green]\N{check mark} Found output for step [bold]"%s"[/] in cache...[/]',
                 step_name,
             )
-            return
+            return None
+
+        step.log_starting()
 
         # Initialize experiment and task spec.
         spec = self._build_experiment_spec(step_graph, step_name)
@@ -427,55 +500,39 @@ class BeakerExecutor(Executor):
             f"{Constants.STEP_EXPERIMENT_PREFIX}{step.unique_id}-{str(uuid.uuid4())[:8]}"
         )
         experiment = self.beaker.experiment.create(experiment_name, spec)
+        experiment_url = self.beaker.experiment.url(experiment)
         cli_logger.info(
-            'Submitted Beaker experiment [cyan]%s[/] for step [b]"%s"[/] (%s)',
-            self.beaker.experiment.url(experiment),
+            '[blue]\N{black rightwards arrow} Submitted Beaker experiment [b]%s[/] for step [b]"%s"[/]...[/]',
+            experiment_url,
             step_name,
-            step.unique_id,
         )
 
         # Follow the experiment and stream the logs until it completes.
-        setup_stage: bool = True
         try:
-            for line in self.beaker.experiment.follow(experiment, strict=True):
-                self._check_if_cancelled()
-                # Every log line from Beaker starts with an RFC 3339 UTC timestamp
-                # (e.g. '2021-12-07T19:30:24.637600011Z'). We don't want to print
-                # the timestamps so we split them off like this:
-                line = line[line.find(b"Z ") + 2 :]
-                line_str = line.decode(errors="ignore").rstrip()
-                if not line_str:
-                    continue
-
-                # Try parsing a JSON log record from the line.
-                try:
-                    log_record = log_record_from_json(line_str)
-                    setup_stage = False
-                    logging.getLogger(log_record.name).handle(log_record)
-                except JSONDecodeError:
-                    if setup_stage:
-                        if line_str.startswith("[TANGO] "):
-                            logger.info(
-                                "[step %s] [setup] %s", step_name, line_str[len("[TANGO] ") :]
-                            )
-                        else:
-                            logger.debug("[step %s] [setup] %s", step_name, line_str)
-                    else:
-                        logger.info("[step %s] %s", step_name, line_str)
-        except JobFailedError:
-            raise ExecutorError(
-                f"Beaker job for step '{step_name}' failed. "
-                f"You can check the logs at {self.beaker.experiment.url(experiment)}"
+            self.beaker.experiment.wait_for(experiment, strict=True, quiet=True, poll_interval=2.0)
+        except (JobFailedError, TaskStoppedError):
+            cli_logger.error(
+                '[red]\N{ballot x} Step [b]"%s"[/] failed. You can check the logs at [b]%s[/][/]',
+                step_name,
+                experiment_url,
+            )
+            raise StepFailedError(
+                f'Beaker job for step "{step_name}" failed. '
+                f"You can check the logs at {experiment_url}",
+                experiment_url,
             )
         except (KeyboardInterrupt, CancellationError):
             cli_logger.warning(
                 'Stopping Beaker experiment [cyan]%s[/] for step [b]"%s"[/] (%s)',
-                self.beaker.experiment.url(experiment),
+                experiment_url,
                 step_name,
                 step.unique_id,
             )
             self.beaker.experiment.stop(experiment)
             raise
+        else:
+            step.log_finished()
+            return experiment_url
 
     @staticmethod
     def _parse_git_remote(url: str) -> Tuple[str, str]:
@@ -562,18 +619,32 @@ class BeakerExecutor(Executor):
         return dataset
 
     def _ensure_cluster(self, task_resources: TaskResources) -> str:
-        cluster_to_use: str
+        cluster_to_use: Optional[str] = None
         if not self.clusters:
             raise ConfigurationError("At least one cluster is required in 'clusters'")
         elif len(self.clusters) == 1:
             cluster_to_use = self.clusters[0]
         else:
+
+            def recency_ranking(cluster_name: str):
+                try:
+                    return self._latest_clusters_used.index(cluster_name)
+                except ValueError:
+                    return -1
+
             available_clusters = sorted(
                 self.beaker.cluster.filter_available(task_resources, *self.clusters),
-                key=lambda x: x.queued_jobs,
+                key=lambda x: (x.queued_jobs, recency_ranking(x.cluster.full_name)),
             )
+
             if available_clusters:
                 cluster_to_use = available_clusters[0].cluster.full_name
+                # Move cluster to the end of `self._latest_clusters_used`
+                try:
+                    self._latest_clusters_used.remove(cluster_to_use)  # type: ignore
+                except ValueError:
+                    pass
+                self._latest_clusters_used.append(cluster_to_use)  # type: ignore
                 logger.debug(f"Using cluster '{cluster_to_use}'")
             else:
                 cluster_to_use = self.clusters[0]
@@ -582,9 +653,15 @@ class BeakerExecutor(Executor):
                     "Will use '%' anyway.",
                     cluster_to_use,
                 )
+
+        if cluster_to_use is None:
+            cluster_to_use = self.clusters[0]
+
         return cluster_to_use
 
     def _build_experiment_spec(self, step_graph: StepGraph, step_name: str) -> ExperimentSpec:
+        from tango.common.logging import TANGO_LOG_LEVEL
+
         step = step_graph[step_name]
         sub_graph = step_graph.sub_graph(step_name)
         step_info = self.workspace.step_info(step)
@@ -614,7 +691,14 @@ class BeakerExecutor(Executor):
         except ValueError:
             raise ExecutorError("BeakerExecutor requires a git repository with a GitHub remote.")
         git_ref = git.commit
-        logger.info("Using GitHub repository '%s/%s' @ %s", github_account, github_repo, git_ref)
+        cli_logger.info(
+            "[blue]\N{black rightwards arrow} Using source code from "
+            '[b]https://github.com/%s/%s/commit/%s[/] for step [b]"%s"[/][/]',
+            github_account,
+            github_repo,
+            git_ref,
+            step_name,
+        )
         self._check_if_cancelled()
 
         # Ensure dataset with the entrypoint script exists and get it.
@@ -646,7 +730,7 @@ class BeakerExecutor(Executor):
         ]
         if self.include_package is not None:
             for package in self.include_package:
-                command += ["-i", package]
+                command += ["-i", package, "--log-level", TANGO_LOG_LEVEL or "debug"]
 
         # Get cluster to use.
         cluster = self._ensure_cluster(task_resources)
@@ -670,6 +754,7 @@ class BeakerExecutor(Executor):
                 resources=task_resources,
                 datasets=self.datasets,
                 env_vars=self.env_vars,
+                priority=self.priority,
             )
             .with_env_var(name="TANGO_VERSION", value=VERSION)
             .with_env_var(name="GITHUB_TOKEN", secret=self.GITHUB_TOKEN_SECRET_NAME)
