@@ -13,6 +13,7 @@ from typing import Any, Dict, List, Optional, Set, Tuple
 import pytz
 
 from .common.from_params import FromParams
+from .common.logging import log_exception
 from .common.util import jsonify, local_timezone, replace_steps_with_unique_id
 from .step import Step
 from .version import VERSION
@@ -31,7 +32,8 @@ def get_pip_packages() -> Optional[List[Tuple[str, str]]]:
 
         return sorted([(d.key, d.version) for d in iter(pkg_resources.working_set)])
     except Exception as exc:
-        logger.exception("Error saving pip packages: %s", exc)
+        logger.error("Error saving pip packages")
+        log_exception(exc)
     return None
 
 
@@ -69,34 +71,15 @@ class GitMetadata(FromParams):
 
     @classmethod
     def check_for_repo(cls) -> Optional["GitMetadata"]:
-        import subprocess
+        from git import InvalidGitRepositoryError, Repo
 
         try:
-            commit = (
-                subprocess.check_output("git rev-parse HEAD".split(" "), stderr=subprocess.DEVNULL)
-                .decode("ascii")
-                .strip()
-            )
-            remote: Optional[str] = None
-            for line in (
-                subprocess.check_output("git remote -v".split(" "))
-                .decode("ascii")
-                .strip()
-                .split("\n")
-            ):
-                remotes: Dict[str, str] = {}
-                if line.endswith("(fetch)"):
-                    name, info = line.split("\t")
-                    url = info.split(" ")[0]
-                    remotes[name] = url
-                if "origin" in remotes:
-                    remote = remotes["origin"]
-                elif remotes:
-                    remote = list(remotes.values())[0]
-            return cls(commit=commit, remote=remote)
-        except (subprocess.CalledProcessError, FileNotFoundError):
+            repo = Repo(".")
+        except InvalidGitRepositoryError:
             return None
 
+        return cls(commit=str(repo.commit()), remote=repo.remote().url)
+
 
 @dataclass
 class TangoMetadata(FromParams):
@@ -245,6 +228,12 @@ class StepInfo(FromParams):
     The raw config of the step.
     """
 
+    metadata: Optional[Dict[str, Any]] = None
+    """
+    Metadata from the step. This comes from the ``step_metadata``
+    argument to the :class:`~tango.step.Step` class.
+    """
+
     platform: PlatformMetadata = field(default_factory=PlatformMetadata)
     """
     The :class:`PlatformMetadata`.
@@ -337,5 +326,13 @@ class StepInfo(FromParams):
             dependencies={dep.unique_id for dep in step.dependencies},
             cacheable=step.cache_results,
             config=replace_steps_with_unique_id(config),
+            metadata=step.metadata,
             **kwargs,
         )
+
+    def refresh(self):
+        """
+        Refresh environment and platform metadata.
+        """
+        self.platform = PlatformMetadata()
+        self.environment = EnvironmentMetadata()
